import os as __os
from typing import Callable, Optional
from . import path as __ospath

FilePredicate = Callable[[__os.DirEntry], bool]


def __scan(__path: str, __where: Optional[FilePredicate]):
    return (
        (file for file in __os.scandir(__path))
        if __where is None
        else (file for file in __os.scandir(__path) if __where(file))
    )


def __count(__path, __where: Optional[FilePredicate] = None):
    result = 0
    for _ in __scan(__path, __where):
        result += 1
    return result


def __remove_path(__path: str | __os.DirEntry):
    path = __path.path if isinstance(__path, __os.DirEntry) else __path
    if __os.path.exists(path):
        __os.remove(path)


def __dirfiles(
    __path: str, __where: Optional[FilePredicate] = None, abs_paths: bool = True
):
    predicate = lambda __file: __ospath.isfile(__ospath.join(__path, __file))
    if __where is not None:
        base = predicate
        predicate = lambda __file: __where(__file) and base(__file)

    result = [file for file in __os.listdir(__path) if predicate(file)]

    if abs_paths:
        result = [__ospath.join(__path, file) for file in result]
    return result


def __mostrecentpath(__path: str, __where: Optional[FilePredicate] = None):
    return max(__dirfiles(__path, __where), key=__ospath.created_at)


def __oldestpath(__path: str, __where: Optional[FilePredicate] = None):
    return min(__dirfiles(__path, __where), key=__ospath.created_at)


def __scan_sorted_by_created_at(
    __path: str, __where: Optional[FilePredicate], abs_paths: bool, reverse: bool
):
    filesin = __dirfiles(__path, __where, abs_paths=abs_paths)
    filesin.sort(key=__ospath.created_at, reverse=reverse)
    return iter(filesin)


def __scan_oldest_to_mostrecent(
    __path: str, __where: Optional[FilePredicate], abs_paths: bool
):
    return __scan_sorted_by_created_at(
        __path, __where, abs_paths=abs_paths, reverse=False
    )


def __scan_mostrecent_to_oldest(
    __path: str, __where: Optional[FilePredicate], abs_paths: bool
):
    return __scan_sorted_by_created_at(
        __path, __where, abs_paths=abs_paths, reverse=True
    )


def count(__path, __where: Optional[FilePredicate] = None):
    return __count(__path, __where)


def remove_path(__path: str | __os.DirEntry):
    return __remove_path(__path)


def scan(__path: str, __where: Optional[FilePredicate] = None):
    return __scan(__path, __where)


def scan_oldest_to_mostrecent(
    __path: str, __where: Optional[FilePredicate] = None, abs_paths: bool = True
):
    return __scan_oldest_to_mostrecent(__path, __where, abs_paths=abs_paths)


def scan_mostrecent_to_oldest(
    __path: str, __where: Optional[FilePredicate] = None, abs_paths: bool = True
):
    return __scan_mostrecent_to_oldest(__path, __where, abs_paths=abs_paths)


def mostrecentpath(__path: str):
    return __mostrecentpath(__path)


def oldestpath(__path: str):
    return __oldestpath(__path)



import os as _os
from typing import Callable


def system(__command: str):
    return _os.system(command=__command)


def join_paths(__name, branch):
    return _os.path.join(__name, branch)


def abspath(__name: str, is_abspath: bool = False):
    return __name if is_abspath else _os.path.abspath(__name)


def basename(__name: str):
    return __name.split("/")[-1] if "/" in __name else __name


def scandir(__name: str, is_abspath: bool, where: Callable[[object], bool] = None):
    result = _os.scandir(__name, is_abspath=is_abspath)
    if where is not None:
        result = (file for file in result if where(file))
    return result


def parentdir(__name: str, is_abspath: bool = False):
    result = abspath(__name, is_abspath=is_abspath)
    if "/" in result:
        components = result.split("/")[:-1]
        result = "/".join(components)
    return result



from . import paths as _paths
from json import dumps as _dumps


def string(
    __name: str,
    content: str,
    is_abspath: bool = False,
    errors: str = "ignore",
    encoding: str = "utf-8",
    exist_ok: bool = True,
):
    path = _paths.safepath(__name, is_abspath=is_abspath, exist_ok=exist_ok)
    with open(path, "w+", errors=errors, encoding=encoding) as file:
        file.write(content)
        file.close()


def bytes(
    __name: str,
    content: bytes,
    is_abspath: bool = False,
    errors: str = "ignore",
    encoding: str = "utf-8",
    exist_ok: bool = True,
):
    path = _paths.safepath(__name, is_abspath=is_abspath, exist_ok=exist_ok)
    with open(path, "w+", errors=errors, encoding=encoding) as file:
        file.write(content)
        file.close()


def json(
    __name: str,
    content,
    indent: int = 4,
    is_abspath: bool = False,
    exist_ok: bool = True,
):
    path = _paths.safepath(__name, is_abspath=is_abspath, exist_ok=exist_ok)
    s = _dumps(content, indent=indent)
    return string(path, content=s, is_abspath=True)


def data(
    __name: str,
    content,
    indent: int = 4,
    is_abspath: bool = False,
    errors: str = "ignore",
    encoding: str = "utf-8",
    exist_ok: bool = True,
):
    path = _paths.safepath(__name, is_abspath=is_abspath, exist_ok=exist_ok)
    if isinstance(content, str):
        result = string(
            path,
            content=content,
            is_abspath=True,
            errors=errors,
            encoding=encoding,
        )
    else:
        result = json(
            path, content=content, indent=indent, is_abspath=True, exist_ok=exist_ok
        )
    return result



from collections import defaultdict as _defaultdict
import os as _os
from . import base as __base
from . import exists_ as __exists


def safepath(__name: str, is_abspath: bool = False, exist_ok: bool = True):
    path = __base.abspath(__name, is_abspath)
    dirname = __base.parentdir(path, True)
    if not __exists.dir(dirname, is_abspath=True):
        _os.makedirs(dirname, exist_ok=exist_ok)
    return path


def filetree(__ft: str, parentdir: str = None, indent: int = 4) -> list[str]:
    paths: list[str] = []
    dirlog = _defaultdict(list[str])
    for line in (line for line in __ft.splitlines(keepends=False) if line):
        offset, path = len(line) - len(line.lstrip()), line.strip()
        dirlog[offset].append(path)
        while offset > 0:
            if dirlog[offset - indent]:
                path = f"{dirlog[offset-indent][-1]}/{path}"
            offset -= indent
        if path:
            paths.append(path)
    if parentdir is not None and parentdir:
        for (i, path) in enumerate(paths):
            if len(path) - len(path.lstrip()) == 0:
                paths[i] = __base.join_paths(parentdir, path)
    return paths



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['osiotk']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'osiotk',
    'version': '1.1.8',
    'description': 'os io toolkit',
    'long_description': '\n## Osiotk\n\n## Description:\n\n    os io toolkit\n\n## Commands:\n\n# Install Poetry:\n#    python3 -m pip install poetry\n\n# Build Package:\n#    python3 -m poetry build\n\n# Publish Package:\n#    python3 -m poetry publish -u <username> -p <password>\n\n# Install Package Locally:\n#    python3 -m pip install osiotk==1.1.8\n\n# Build And Publish Package:\n#    python3 -m poetry build;python3 -m poetry publish -u <username> -p <password>\n\n# Build Package; Publish Package; Install Package:\n#    python3 -m poetry build;python3 -m poetry publish -u <username> -p <password>;python3 -m pip install osiotk==1.1.8\n\n# Test Osiotk:\n#    python3 -m pytest -rpP tests/\n\n',
    'author': 'wayfaring-stranger',
    'author_email': 'zw6p226m@duck.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os as _os
from . import base as __base
from . import is_ as __is


def node(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    return _os.path.exists(path)


def file(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    return node(path, is_abspath=True) and __is.file(path, is_abspath=True)


def dir(__name: str, is_abspath: bool = False):
    path = __base.abspath(__name, is_abspath)
    return node(path, is_abspath=True) and __is.dir(path, is_abspath=True)
